<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Image_Graph - PEAR PHP OO Graph Rendering Utility.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version. This library is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details. You should have received a copy of
 * the GNU Lesser General Public License along with this library; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307 USA
 *
 * @category   Images
 * @package    Image_Graph
 * @subpackage Figure
 * @author     Jesper Veggerby <pear.nosey@veggerby.dk>
 * @copyright  Copyright (C) 2003, 2004 Jesper Veggerby Hansen
 * @license    http://www.gnu.org/copyleft/lesser.html  LGPL License 2.1
 * @version    CVS: $Id: Rectangle.php,v 1.7 2005/02/21 20:49:55 nosey Exp $
 * @link       http://pear.php.net/package/Image_Graph
 */

/**
 * Include file Image/Graph/Element.php
 */
require_once 'Image/Graph/Element.php';

/**
 * Rectangle to draw on the canvas
 *
 * @category   Images
 * @package    Image_Graph
 * @subpackage Figure
 * @author     Jesper Veggerby <pear.nosey@veggerby.dk>
 * @copyright  Copyright (C) 2003, 2004 Jesper Veggerby Hansen
 * @license    http://www.gnu.org/copyleft/lesser.html  LGPL License 2.1
 * @version    Release: @package_version@
 * @link       http://pear.php.net/package/Image_Graph
 */
class Image_Graph_Figure_Rectangle extends Image_Graph_Element
{

    /**
     * Rectangle [Construcor]
     *
     * @param int $x The leftmost pixel of the box on the canvas
     * @param int $y The topmost pixel of the box on the canvas
     * @param int $width The width in pixels of the box on the canvas
     * @param int $height The height in pixels of the box on the canvas
     */
    function &Image_Graph_Figure_Rectangle($x, $y, $width, $height)
    {
        parent::Image_Graph_Element();
        $this->_setCoords($x, $y, $x + $width, $y + $height);
    }

    /**
     * Output the box
     *
     * @return bool Was the output 'good' (true) or 'bad' (false).
     * @access private
     */
    function _done()
    {
        if (parent::_done() === false) {
            return false;
        }

        $this->_driver->startGroup(get_class($this));
        
        $this->_getFillStyle();
        $this->_getLineStyle();
        $this->_driver->rectangle(
            $this->_left,
            $this->_top,
            $this->_right,
            $this->_bottom
        );
        
        $this->_driver->endGroup();
        
        return true;
    }

}
?>